// C Source File
// Created 5/9/2003; 6:47:52 PM
/*
	This is the source code for the adaptation of the Monty Hall game.
	It is a very simple program meant to help beginner C programmers.
*/

#define USE_TI89              // Compile for TI-89
#define USE_TI92PLUS          // Compile for TI-92 Plus
#define USE_V200              // Compile for V200

#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization

#define MIN_AMS 100           // Compile for AMS 1.00 or higher

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files

//Here are the sprite definitions
static unsigned short int door[] = {
	0xffff, 0xc003, 0xc003, 0xc003, 0xc003,
	0xc003, 0xc003, 0xc003, 0xc003, 0xc003,
	0xc01b, 0xc01b, 0xc003, 0xc003, 0xc003,
	0xc003, 0xc003, 0xc003, 0xc003, 0xffff
};
static unsigned short int shoe[] = {
	0x0000, 0x0000, 0x0000, 0x0000, 0x03c0,
	0x0620, 0x0420, 0x07e0, 0x0ea0, 0x0aa0,
	0x1ba0, 0x1ab0, 0x1bb0, 0x1830, 0x0c60,
	0x0680, 0x0100, 0x0000, 0x0000, 0x0000
};
static unsigned short int car[] = {
	0x0000, 0x0000, 0x0000, 0x0070, 0x0188,
	0x01c4, 0x02a2, 0x0492, 0x0d4a, 0x3a3c,
	0x294c, 0x4490, 0x4260, 0x41c0, 0x2080,
	0x1180, 0x0e00, 0x0000, 0x0000, 0x0000
};

// Main Function
void _main(void)
{
	int x1 = 52, x2 = 72, x3 = 92;
	int y = 40;
	int *x[] = {&x1, &x2, &x3};
	//Current keeps track of the selected door; on tells whether the flashing sprite is on or off;
	//prize is the position of the car; done is used to tell whether to end the current game or not;
	//key stores the users input; numdoors keeps track of the number of closed doors;
	//open is the position of the open door; and i is just used in for loops
	int current, on, prize, done, key, numdoors, open, i;
	//prev will store the calc's default font
	short int prev = 0;
	
	//This loop repeats the games until the user presses ESC
	do {
		done = key = current = 0;
		on = 1;
		numdoors = 3;
		open = -2;
		
		ClrScr();
	
		DrawStr(10, 0, "The Monty Hall Game!", A_NORMAL);
		//set font and store previous font
		prev = FontSetSys(F_4x6);
		DrawStr(0, 10, "Use the arrow keys and enter to play.", A_NORMAL);
		DrawStr(0, 20, "Guess the door that has the car behind it.", A_NORMAL);
		FontSetSys(prev);
	
		randomize();
		prize = random(3);
	
		//Display the doors
		for (i = 0; i < 3; i++) {
			Sprite16(*x[i], y, 20, door, LCD_MEM, SPRT_XOR);
		}
		//This loop continues the current game
		while (!done)
		{
		
			//Prepare timer
			OSFreeTimer(USER_TIMER);
			OSRegisterTimer(USER_TIMER, 10);
			
			//This loop repeats every half second or every key hit
			while (!OSTimerExpired(USER_TIMER) && !kbhit());
		
				//flash current door
				on = !on;
				Sprite16(*x[current], y, 20, door, LCD_MEM, SPRT_XOR);
		
				if (kbhit()) {
					key = ngetchx();
				} else {
					key = 0;
				}
					//check the key hits
					if (key == KEY_ESC) {
						done = 1;
					}	
					else if (key == KEY_ENTER) {
						if (numdoors == 3) {
							//find a door to open
							for(i = 0; i < 3; i++) {
								if (i == prize || i == current) continue;
								//open the door
								else {
									Sprite16(*x[i], y, 20, door, LCD_MEM, SPRT_XOR);
									Sprite16(*x[i], y, 20, shoe, LCD_MEM, SPRT_XOR);
									numdoors = 2;
									open = i;
									break;
								}
							}
						}
						else {
							if (on) {
								Sprite16(*x[current], y, 20, door, LCD_MEM, SPRT_XOR); 
							}	
							//winner
							if (current == prize) {
								Sprite16(*x[current], y, 20, car, LCD_MEM, SPRT_XOR);
								DrawStr(0, 90, "You won the car!", A_NORMAL);
							}
							//loser
							else {
								Sprite16(*x[current], y, 20, shoe, LCD_MEM, SPRT_XOR);
								DrawStr(0, 90, "You lose...", A_NORMAL);
							}
							done = 1;
						}
					}
					//This chunk should be pretty straight-forward. If not, read it carefully.
					else if (key == KEY_LEFT) {
						if(!on) {
							Sprite16(*x[current], y, 20, door, LCD_MEM, SPRT_XOR);
							on = 1;
							}
						current--;
						if (current < 0) {
							current++;
						}
						else if (current == open) {
							current--;
							if (current < 0) current += 2;
						}
						Sprite16(*x[current], y, 20, door, LCD_MEM, SPRT_XOR);
						on = 0;
					}
					else if (key == KEY_RIGHT) {
						if(!on) {
							Sprite16(*x[current], y, 20, door, LCD_MEM, SPRT_XOR);
							on = 1;
						}
						current++;
						if (current > 2) {
							current--;
						}
						else if (current == open) {
							current++;
							if (current > 2) current -= 2;
						}
						Sprite16(*x[current], y, 20, door, LCD_MEM, SPRT_XOR);
						on = 0;
					}
		}
		FontSetSys(F_4x6);
		DrawStr(0, 70, "Press any key to continue; ESC to quit.", A_NORMAL);
		FontSetSys(prev);
		key = ngetchx();
	} while (key != KEY_ESC);
}